@@ -30,6 +30,7 @@
 #include "../pvar.h"
 #include "../timer.h"
 #include "../ut.h"
+#include "../ipc.h"
 
 
 int events_no = 0;
@@ -103,30 +104,19 @@ int evi_raise_event(event_id_t id, evi_params_t* params)
 	 * because these might be nested, a different message has
 	 * to be generated each time
 	 */
-	req = (struct sip_msg*)pkg_malloc(sizeof(struct sip_msg));
+	req = get_dummy_sip_msg();
 	if(req == NULL)
 	{
 		LM_ERR("No more memory\n");
 		return -1;
 	}
-	memset(req, 0, sizeof(struct sip_msg));
-
-	req->first_line.type = SIP_REQUEST;
-	req->first_line.u.request.method.s= "DUMMY";
-	req->first_line.u.request.method.len= 5;
-	req->first_line.u.request.uri.s= "sip:user@domain.com";
-	req->first_line.u.request.uri.len= 19;
-	req->rcv.src_ip.af = AF_INET;
-	req->rcv.dst_ip.af = AF_INET;
-
 
 	bak_avps = set_avp_list(&event_avps);
 
 	status = evi_raise_event_msg(req, id, params);
 
 	/* clean whatever extra structures were added by script functions */
-	free_sip_msg(req);
-	pkg_free(req);
+	release_dummy_sip_msg(req);
 
 	/* remove all avps added */
 	destroy_avp_list(&event_avps);
@@ -134,22 +124,68 @@ int evi_raise_event(event_id_t id, evi_params_t* params)
 
 	return status;
 }
+/* this function checks the subscribers of an event and remove them if 
+they are past their expiry - dont want to print expired events as well */
+void evi_remove_expired_subs(event_id_t id) {
+	evi_subs_p subs, prev;
+	long now;
 
+	lock_get(events[id].lock);
+	now = time(0);
+	subs = events[id].subscribers;
+	prev = NULL;
+	while (subs) {
+		if (!subs->reply_sock) {
+			LM_ERR("unknown destination\n");
+			goto next_sub;
+		}
+		/* check expire */
+		if (!(subs->reply_sock->flags & EVI_PENDING) &&
+			subs->reply_sock->flags & EVI_EXPIRE &&
+			subs->reply_sock->subscription_time +
+			subs->reply_sock->expire < now) {
+			LM_DBG("removing expired subscriber %.*s:%.*s:%d for event %.*s\n",
+				subs->trans_mod->proto.len,subs->trans_mod->proto.s,
+				subs->reply_sock->address.len, subs->reply_sock->address.s,
+				subs->reply_sock->port,events[id].name.len,events[id].name.s);
+			if (subs->trans_mod && subs->trans_mod->free)
+				subs->trans_mod->free(subs->reply_sock);
+			else
+				shm_free(subs->reply_sock);
+			if (!prev) {
+				events[id].subscribers = subs->next;
+				shm_free(subs);
+				subs = events[id].subscribers;
+			} else {
+				prev->next = subs->next;
+				shm_free(subs);
+				subs = prev->next;
+			}
+			continue;
+		}
+next_sub:
+		prev = subs;
+		subs = subs->next;
+	}
+	lock_release(events[id].lock);
+}
+/* XXX: this function should release its parameters before exiting */
 int evi_raise_event_msg(struct sip_msg *msg, event_id_t id, evi_params_t* params)
 {
 	evi_subs_p subs, prev;
+	evi_async_ctx_t evi_async_status = {NULL, NULL};
 	long now;
 	int flags, pflags = 0;
 	int ret = 0;
 
 	if (id < 0 || id >= events_no) {
 		LM_ERR("invalid event %d\n", id);
-		return -1;
+		goto free;
 	}
 
 	if (events_rec_level == 0) {
 		LM_ERR("Too many nested events %d\n", MAX_REC_LEV);
-		return -1;
+		goto free;
 	}
 	events_rec_level--;
 	if (params)
@@ -204,7 +240,7 @@ int evi_raise_event_msg(struct sip_msg *msg, event_id_t id, evi_params_t* params
 		lock_release(events[id].lock);
 
 		ret += (subs->trans_mod->raise)(msg, &events[id].name,
-					subs->reply_sock, params);
+					subs->reply_sock, params, &evi_async_status);
 
 		lock_get(events[id].lock);
 		subs->reply_sock->flags = flags;
@@ -214,14 +250,16 @@ next:
 	}
 	lock_release(events[id].lock);
 
+	events_rec_level++;
+	if (params)
+		params->flags = pflags;
+free:
 	/* done sending events - free parameters */
 	if (params) {
 		/* make sure no one is messing with our flags */
-		params->flags = pflags;
 		if (params->flags & EVI_FREE_LIST)
 			evi_free_params(params);
 	}
-	events_rec_level++;
 	return ret;
 
 }
@@ -273,7 +311,7 @@ int evi_event_subscribe(str event_name,
 {
 	evi_subs_t *subscriber = NULL;
 	evi_event_p event;
-	evi_export_t *trans_mod = NULL;
+	const evi_export_t *trans_mod = NULL;
 	evi_reply_sock *sock;
 
 	event = evi_get_event(&event_name);
@@ -430,106 +468,106 @@ error:
 	return -1;
 }
 
-struct mi_root * mi_event_subscribe(struct mi_root *root, void *param )
+
+static mi_response_t *mi_event_subscribe(const mi_params_t *params, unsigned int expire)
 {
-	struct mi_node *node;
 	int ret;
-	unsigned int expire = 0;
 	str event_name, transport_sock;
 
-	/* event name */
-	node = root->node.kids;
-	if (!node || !node->value.len || !node->value.s) {
-		LM_ERR("no parameters received\n");
-		goto missing_param;
-	}
-	event_name = node->value;
+	if (get_mi_string_param(params, "event", &event_name.s,
+		&event_name.len) < 0 || !event_name.s || !event_name.len)
+		return init_mi_param_error();
 
-	/* socket */
-	node = node->next;
-	if (!node || !node->value.len || !node->value.s) {
-		LM_ERR("no transport type\n");
-		goto missing_param;
-	}
-	transport_sock = node->value;
-
-	/* check expire */
-	node = node->next;
-	if (node) {
-		/* expiration period is set */
-		if (str2int(&node->value, &expire) < 0) {
-			LM_ERR("invalid expire value %.*s", node->value.len, node->value.s);
-			goto bad_param;
-		}
-	} else
-		expire = DEFAULT_EXPIRE;
+	if (get_mi_string_param(params, "socket", &transport_sock.s,
+		&transport_sock.len) < 0 || !transport_sock.s || !transport_sock.len)
+		return init_mi_param_error();
 
 	ret = evi_event_subscribe(event_name, transport_sock, expire, 1);
 	if (ret < 0)
-		goto bad_param;
-	return ret ? init_mi_tree(200, MI_SSTR(MI_OK)) : 0;
+		return init_mi_error(400, MI_SSTR("Bad parameter value"));
 
-missing_param:
-	return init_mi_tree( 400, MI_SSTR(MI_MISSING_PARM));
-
-bad_param:
-	return init_mi_tree( 400, MI_SSTR(MI_BAD_PARM));
+	return ret ? init_mi_result_ok() : 0;
 }
 
+mi_response_t *w_mi_event_subscribe(const mi_params_t *params,
+								struct mi_handler *async_hdl)
+{
+	return mi_event_subscribe(params, DEFAULT_EXPIRE);
+}
+
+mi_response_t *w_mi_event_subscribe_1(const mi_params_t *params,
+								struct mi_handler *async_hdl)
+{
+	int expire;
+
+	if (get_mi_int_param(params, "expire", &expire) < 0)
+		return init_mi_param_error();
+
+	if (expire < 0)
+		return init_mi_error_extra(JSONRPC_INVAL_PARAMS_CODE,
+			MI_SSTR(JSONRPC_INVAL_PARAMS_MSG),
+			MI_SSTR("Negative expire value"));
+
+	return mi_event_subscribe(params, expire);
+}
 
 
 /* used to list all the registered events */
-struct mi_root * mi_events_list(struct mi_root *cmd_tree, void *param)
+mi_response_t *mi_events_list(const mi_params_t *params,
+								struct mi_handler *async_hdl)
 {
-	struct mi_root *rpl_tree;
-	struct mi_node *node=NULL, *rpl=NULL;
+	mi_response_t *resp;
+	mi_item_t *resp_obj;
+	mi_item_t *events_arr, *event_item;
 	unsigned i;
 
-	rpl_tree = init_mi_tree( 200, MI_OK_S, MI_OK_LEN);
-	if (rpl_tree==0)
+	resp = init_mi_result_object(&resp_obj);
+	if (!resp)
 		return 0;
-	rpl = &rpl_tree->node;
-	rpl->flags |= MI_IS_ARRAY;
+
+	events_arr = add_mi_array(resp_obj, MI_SSTR("Events"));
+	if (!events_arr) {
+		free_mi_response(resp);
+		return 0;
+	}
 
 	for (i = 0; i < events_no; i++) {
-		node = add_mi_node_child(rpl, 0, "Event", 5,
-				events[i].name.s, events[i].name.len);
-		if(node == NULL)
+		event_item = add_mi_object(events_arr, NULL, 0);
+		if (!event_item)
 			goto error;
 
-		if (!addf_mi_attr(node, 0, "id", 2, "%d", events[i].id))
+		if (add_mi_string(event_item, MI_SSTR("name"),
+			events[i].name.s, events[i].name.len) < 0)
 			goto error;
 
-		if ((i + 1) % 50 == 0) {
-			flush_mi_tree(rpl_tree);
-		}
+		if (add_mi_number(event_item, MI_SSTR("id"), events[i].id) < 0)
+			goto error;
 	}
-	return rpl_tree;
+
+	return resp;
+
 error:
-	free_mi_tree(rpl_tree);
+	free_mi_response(resp);
 	return 0;
 }
 
-static int evi_print_subscriber(struct mi_node *rpl, evi_subs_p subs)
+static int evi_print_subscriber(mi_item_t *subs_obj, evi_subs_p subs)
 {
 	evi_reply_sock *sock;
-	struct mi_node *node;
 	str socket;
+	long now;
+	int expiry_countdown;
 
 	if (!subs || !subs->trans_mod || !subs->trans_mod->print) {
 		LM_ERR("subscriber does not have a print method exported\n");
 		return -1;
 	}
 
-	node = add_mi_node_child(rpl, 0, "Subscriber", 10, 0, 0);
-	if(node == NULL)
-		return -1;
-
 	sock = subs->reply_sock;
 	if (!sock) {
 		LM_DBG("no socket specified\n");
-		if (!add_mi_attr(node, 0, "protocol", 8,
-				subs->trans_mod->proto.s, subs->trans_mod->proto.len))
+		if (add_mi_string(subs_obj, MI_SSTR("protocol"),
+			subs->trans_mod->proto.s, subs->trans_mod->proto.len) < 0)
 			return -1;
 		return 0;
 	}
@@ -537,16 +575,27 @@ static int evi_print_subscriber(struct mi_node *rpl, evi_subs_p subs)
 	socket = subs->trans_mod->print(sock);
 	LM_DBG("print subscriber socket <%.*s> %d\n",
 			socket.len, socket.s, socket.len);
-	if (!addf_mi_attr(node, MI_DUP_VALUE, "socket", 6, "%.*s:%.*s",
+	if (add_mi_string_fmt(subs_obj, MI_SSTR("socket"), "%.*s:%.*s",
 			subs->trans_mod->proto.len, subs->trans_mod->proto.s,
-			socket.len, socket.s))
+			socket.len, socket.s) < 0)
 		return -1;
-
 	if (sock->flags & EVI_EXPIRE) {
-		if (!addf_mi_attr(node, 0, "expire", 6, "%d", sock->expire))
-			return -1;
+		/* indicate the expiration time used for the event*/
+		if (add_mi_number(subs_obj, MI_SSTR("expire"), sock->expire) < 0)
+				return -1;
+		now = time(0);
+		/* calculate the remaining time for the subscriber to be expired */
+		expiry_countdown = sock->expire - (now - subs->reply_sock->subscription_time);
+		if (expiry_countdown > 0) {
+			if (add_mi_number(subs_obj, MI_SSTR("ttl"), expiry_countdown) < 0)
+				return -1;
+		}else{
+			/* Mark event as expired if time-to-expire is reached*/
+			if (add_mi_string(subs_obj, MI_SSTR("ttl"), MI_SSTR("expired")) < 0)
+				return -1;
+		}
 	} else {
-		if (!add_mi_attr(node, 0, "expire", 6, "never", 5))
+		if (add_mi_string(subs_obj, MI_SSTR("expire"), MI_SSTR("never")) < 0)
 			return -1;
 	}
 	/* XXX - does subscription time make sense? */
@@ -554,44 +603,43 @@ static int evi_print_subscriber(struct mi_node *rpl, evi_subs_p subs)
 	return 0;
 }
 
-struct evi_mi_param {
-	struct mi_node * node;
-	struct mi_root * root;
-	int nr;
-};
-
-
-static int evi_print_event(struct evi_mi_param *param,
-								evi_event_t *ev, evi_subs_p subs)
+static int evi_print_event(mi_item_t *ev_obj, evi_event_t *ev, evi_subs_p subs)
 {
-	struct mi_node *node=NULL;
-	struct mi_node *rpl = param->node;
+	mi_item_t *subs_arr, *subs_item;
 
 	/* add event only if there are subscribers */
 	if (!subs && !ev->subscribers)
 		return 0;
 
-	node = add_mi_node_child(rpl, MI_IS_ARRAY, "Event", 5,
-		ev->name.s, ev->name.len);
-	if(node == NULL)
+	if (add_mi_string(ev_obj, MI_SSTR("name"), ev->name.s, ev->name.len) < 0)
 		goto error;
 
-	if (!addf_mi_attr(node, 0, "id", 2, "%d", ev->id))
+	if (add_mi_number(ev_obj, MI_SSTR("id"), ev->id) < 0)
 		goto error;
 
 	if (subs) {
-		if (evi_print_subscriber(node, subs) < 0) {
+		subs_item = add_mi_object(ev_obj, MI_SSTR("Subscriber"));
+		if (!subs_item)
+			goto error;
+
+		if (evi_print_subscriber(subs_item, subs) < 0) {
 			LM_ERR("cannot print subscriber info\n");
 			goto error;
 		}
 	} else {
+		subs_arr = add_mi_array(ev_obj, MI_SSTR("Subscribers"));
+		if (!subs_arr)
+			goto error;
+
 		for (subs = ev->subscribers; subs; subs = subs->next) {
-			if (evi_print_subscriber(node, subs) < 0) {
+			subs_item = add_mi_object(subs_arr, NULL, 0);
+			if (!subs_item)
+				goto error;
+
+			if (evi_print_subscriber(subs_item, subs) < 0) {
 				LM_ERR("cannot print subscriber info\n");
 				goto error;
 			}
-			if (++param->nr % 50 == 0)
-				flush_mi_tree(param->root);
 		}
 	}
 	return 0;
@@ -602,7 +650,7 @@ error:
 
 static evi_subs_p evi_get_subscriber(evi_event_p event, str sock_str)
 {
-	evi_export_t * trans_mod;
+	const evi_export_t * trans_mod;
 	evi_subs_p subscriber = NULL;
 	evi_reply_sock * sock;
 
@@ -641,68 +689,324 @@ static evi_subs_p evi_get_subscriber(evi_event_p event, str sock_str)
 	return subscriber;
 }
 
-
-/* used to list all subscribers */
-struct mi_root * mi_subscribers_list(struct mi_root *cmd_tree, void *param)
+static mi_response_t *mi_subscribers_list(evi_event_p event, evi_subs_p subs)
 {
-	struct mi_root *rpl_tree, *err=NULL;
-	struct mi_node *node=NULL, *rpl=NULL;
-	struct evi_mi_param prm;
-	evi_subs_p subs = NULL;
-	evi_event_p event;
-	unsigned i;
+	mi_response_t *resp;
+	mi_item_t *resp_obj;
+	mi_item_t *event_obj;
 
-	rpl_tree = init_mi_tree( 200, MI_OK_S, MI_OK_LEN);
-	if (rpl_tree==0)
+	resp = init_mi_result_object(&resp_obj);
+	if (!resp)
 		return 0;
 
-	memset(&prm, 0, sizeof(struct evi_mi_param));
-
-	rpl = &rpl_tree->node;
-	rpl->flags |= MI_IS_ARRAY;
-	node = cmd_tree->node.kids;
-	prm.node = rpl;
-	prm.root = rpl_tree;
-	/* dump all info */
-	if (!node) {
-		for (i = 0; i < events_no; i++) {
-			if (evi_print_event(&prm, &events[i], NULL) < 0) {
-				LM_ERR("cannot print event %.*s info\n",
-						events[i].name.len, events[i].name.s);
-				goto error;
-			}
-		}
-		return rpl_tree;
-	}
-	/* get the event name */
-	event = evi_get_event(&node->value);
-	if (!event) {
-		err = init_mi_tree(404, MI_SSTR("Event not published"));
+	event_obj = add_mi_object(resp_obj, MI_SSTR("Event"));
+	if (!event_obj)
 		goto error;
-	}
-	node = node->next;
-	/* if a subscriber was specified */
-	if (node) {
-		if (node->next) {
-			err = init_mi_tree(400, MI_SSTR(MI_MISSING_PARM));
-			goto error;
-		}
-		/* search for subscriber */
-		subs = evi_get_subscriber(event, node->value);
-		if (!subs) {
-			err = init_mi_tree(404, MI_SSTR("Subscriber does not exist"));
-			goto error;
-		}
-	}
 
-	if (evi_print_event(&prm, event, subs) < 0) {
-		LM_ERR("cannot print event %.*s info\n", event->name.len, event->name.s);
+	if (evi_print_event(event_obj, event, subs) < 0) {
+		LM_ERR("cannot print event %.*s info\n",
+			event->name.len, event->name.s);
 		goto error;
 	}
 
-	return rpl_tree;
+	return resp;
 
 error:
-	free_mi_tree(rpl_tree);
-	return err;
+	free_mi_response(resp);
+	return NULL;
+}
+
+/* used to list all subscribers */
+mi_response_t *w_mi_subscribers_list(const mi_params_t *params,
+								struct mi_handler *async_hdl)
+{
+	mi_response_t *resp;
+	mi_item_t *resp_obj;
+	mi_item_t *events_arr, *event_item;
+	int i;
+
+	resp = init_mi_result_object(&resp_obj);
+	if (!resp)
+		return 0;
+
+	events_arr = add_mi_array(resp_obj, MI_SSTR("Events"));
+	if (!events_arr)
+		goto error;
+
+	for (i = 0; i < events_no; i++) {
+		/* before printing the subs list check for any expired subscribers and remove them*/
+		evi_remove_expired_subs(events[i].id);
+		if (!events[i].subscribers)
+			continue;
+
+		event_item = add_mi_object(events_arr, NULL, 0);
+		if (!event_item)
+			goto error;
+
+		if (evi_print_event(event_item, &events[i], NULL) < 0) {
+			LM_ERR("cannot print event %.*s info\n",
+				events[i].name.len, events[i].name.s);
+			goto error;
+		}
+	}
+
+	return resp;
+
+error:
+	free_mi_response(resp);
+	return 0;
+}
+
+mi_response_t *w_mi_subscribers_list_1(const mi_params_t *params,
+								struct mi_handler *async_hdl)
+{
+	str event_s;
+	evi_event_p event;
+	int evid;
+	if (get_mi_string_param(params, "event", &event_s.s, &event_s.len) < 0)
+		return init_mi_param_error();
+
+	event = evi_get_event(&event_s);
+	if (!event)
+		return init_mi_error(404, MI_SSTR("Event not published"));
+	/* get the event id & before printing the subs list check for any expired subscribers and remove them*/
+	evid = evi_get_id(&event_s);
+	if (evid == -1)
+		return init_mi_error(404, MI_SSTR("Can't get the id"));
+
+	evi_remove_expired_subs(evid);
+
+	return mi_subscribers_list(event, NULL);
+}
+
+mi_response_t *w_mi_subscribers_list_2(const mi_params_t *params,
+								struct mi_handler *async_hdl)
+{
+	str event_s;
+	str subs_s;
+	evi_event_p event;
+	evi_subs_p subs;
+	int evid;
+	if (get_mi_string_param(params, "event", &event_s.s, &event_s.len) < 0)
+		return init_mi_param_error();
+
+	event = evi_get_event(&event_s);
+	if (!event)
+		return init_mi_error(404, MI_SSTR("Event not published"));
+	/* get the event id & before printing the subs list check for any expired subscribers and remove them*/
+	evid = evi_get_id(&event_s);
+	if (evid == -1)
+		return init_mi_error(404, MI_SSTR("Can't get the id"));
+
+	evi_remove_expired_subs(evid);
+
+	if (get_mi_string_param(params, "socket", &subs_s.s, &subs_s.len) < 0)
+		return init_mi_param_error();
+
+	subs = evi_get_subscriber(event, subs_s);
+	if (!subs)
+		return init_mi_error(404, MI_SSTR("Subscriber does not exist"));
+
+	return mi_subscribers_list(event, subs);
+}
+
+evi_params_p mi_raise_event_json_params(str *params)
+{
+	int err;
+	cJSON *param;
+	cJSON *jparams;
+	str name, jstring;
+	evi_params_p eparams = NULL;
+	char *tmp = pkg_malloc(params->len + 1);
+	if (!tmp) {
+		LM_ERR("could not create temporary buffer!\n");
+		return NULL;
+	}
+	memcpy(tmp, params->s, params->len);
+	tmp[params->len] = 0;
+	jparams = cJSON_Parse(tmp);
+	pkg_free(tmp);
+	if (!jparams) {
+		LM_DBG("could not parse json '%.*s'\n", params->len, params->s);
+		return NULL;
+	} else
+		LM_DBG("treating params as json '%.*s'\n", params->len, params->s);
+
+	if (!(jparams->type &cJSON_Object)) {
+		LM_ERR("params json is not an object\n");
+		return NULL;
+	}
+	/* parse params as json */
+	if (!(eparams = evi_get_params())) {
+		LM_ERR("cannot create parameters list\n");
+		goto error;
+	}
+	for (param = jparams->child; param; param = param->next) {
+		name.s = param->string;
+		name.len = strlen(name.s);
+		switch (param->type) {
+			case cJSON_Number:
+				err = evi_param_add_int(eparams, &name, &param->valueint);
+				break;
+			case cJSON_String:
+				jstring.s = param->valuestring;
+				jstring.len = strlen(jstring.s);
+				err = evi_param_add_str(eparams, &name, &jstring);
+				break;
+			default:
+				jstring.s = cJSON_PrintUnformatted(param);
+				jstring.len = strlen(jstring.s);
+				err = evi_param_add_str(eparams, &name, &jstring);
+				cJSON_PurgeString(jstring.s);
+				break;
+		}
+		if (err) {
+			LM_ERR("could not add parameter %s\n", name.s);
+			goto error_free;
+		}
+	}
+	cJSON_Delete(jparams);
+	return eparams;
+error_free:
+	evi_free_params(eparams);
+error:
+	cJSON_Delete(jparams);
+	return NULL;
+}
+
+evi_params_p mi_raise_event_array_params(mi_item_t *array, int no)
+{
+	int i;
+	str param;
+	evi_params_p eparams = NULL;
+
+	LM_DBG("treating params as array\n");
+
+	/* parse params as json */
+	if (!(eparams = evi_get_params())) {
+		LM_ERR("cannot create parameters list\n");
+		return NULL;
+	}
+
+	for (i = 0; i < no; i++) {
+		if (get_mi_arr_param_string(array, i, &param.s, &param.len) < 0) {
+			LM_ERR("cannot fetch array element %d\n", i);
+			goto error;
+		}
+		if (evi_param_add_str(eparams, NULL, &param)) {
+			LM_ERR("cannot add new params %d\n", i);
+			goto error;
+		}
+	}
+
+	return eparams;
+error:
+	evi_free_params(eparams);
+	return NULL;
+}
+
+struct rpc_raise_event_dispatch {
+	event_id_t id;
+	evi_params_p params;
+};
+
+
+void rpc_raise_event(int sender, void *param)
+{
+	struct rpc_raise_event_dispatch *p = (struct rpc_raise_event_dispatch *)param;
+	if (evi_raise_event(p->id, p->params))
+		LM_ERR("cannot raise event RPC\n");
+	evi_free_shm_params(p->params);
+	shm_free(p);
+}
+
+
+int evi_dispatch_event( event_id_t id, evi_params_p params)
+{
+	evi_params_p sparams = NULL;
+	struct rpc_raise_event_dispatch *djob;
+
+	if (params) {
+		sparams = evi_dup_shm_params(params);
+		evi_free_params(params);
+		if (!sparams) {
+			LM_ERR("could not shm duplicate evi params!\n");
+			goto error;
+		}
+	}
+
+	djob = shm_malloc(sizeof (*djob));
+	if (!djob) {
+		LM_ERR("could not allocate new job!\n");
+		goto error;
+	}
+	djob->id = id;
+	djob->params = sparams;
+
+	if (ipc_dispatch_rpc(rpc_raise_event, djob) < 0) {
+		LM_ERR("could not dispatch raise event job!\n");
+		goto error;
+	}
+
+	return 0;
+error:
+	if (sparams)
+		evi_free_shm_params(sparams);
+	return -1;
+}
+
+
+mi_response_t *w_mi_raise_event(const mi_params_t *params,
+								struct mi_handler *async_hdl)
+{
+	int no;
+	str event_s;
+	str tparams;
+	event_id_t id;
+	mi_item_t *values;
+	evi_params_p eparams = NULL;
+
+	if (get_mi_string_param(params, "event", &event_s.s, &event_s.len) < 0)
+		return init_mi_param_error();
+
+	id = evi_get_id(&event_s);
+	if (id == EVI_ERROR)
+		return init_mi_error(404, MI_SSTR("Event not registered"));
+
+	/* check if there are any subscribers */
+	if (!evi_probe_event(id))
+		return init_mi_error(480, MI_SSTR("Temporarily Unavailable"));
+
+	/* check to see if we have an array params, or key-value one */
+	switch (try_get_mi_array_param(params, "params", &values, &no)) {
+		case -1:
+		case -3:
+			/* no params used */
+			break;
+		case -2:
+			/* not an array - most likely it's a string */
+			if (get_mi_string_param(params, "params", &tparams.s, &tparams.len) < 0)
+				return init_mi_error(400, MI_SSTR("No Params"));
+			eparams = mi_raise_event_json_params(&tparams);
+			if (!eparams)
+				return init_mi_error(400, MI_SSTR("Bad Params"));
+			break;
+		case 0:
+			/* this is an array - push it like this */
+			eparams = mi_raise_event_array_params(values, no);
+			if (!eparams)
+				return init_mi_error(400, MI_SSTR("Bad Params"));
+			break;
+	}
+
+	if (evi_dispatch_event( id, eparams)<0) {
+		LM_ERR("could not dispatch raise event job!\n");
+		goto error;
+	}
+
+	return init_mi_result_ok();
+error:
+	if (eparams)
+		evi_free_params(eparams);
+	return init_mi_error(500, MI_SSTR("Cannot Raise Event"));
 }
