@@ -72,6 +72,15 @@ size_t write_data(char *ptr, size_t size, size_t nmemb, void *stream) {
 	return size * nmemb;
 }
 
+#define w_curl_easy_setopt(h, opt, value) \
+	do { \
+		CURLcode rc = curl_easy_setopt(h, opt, value); \
+		if (rc != CURLE_OK) { \
+			LM_ERR("curl_easy_setopt(%d): (%s)\n", opt, curl_easy_strerror(rc)); \
+			goto error; \
+		} \
+	} while (0)
+
 /* simple FTTP POST using curl lib */
 int post(char*  url, char* xml, char** response){
 	CURL *curl;
@@ -83,16 +92,17 @@ int post(char*  url, char* xml, char** response){
 	data.data = malloc(1024); /* reasonable size initial buffer */
 	if(NULL == data.data) {
 		LM_ERR("NO MEMORY\n");
+		curl_easy_cleanup(curl);
 		return -1;
 	}
 	memset(data.data, '\0', 1024);
 	LM_DBG("CURL PASSOU MALLOC\n");
 
 	if(curl) {
-		curl_easy_setopt(curl, CURLOPT_URL, url);
-		curl_easy_setopt(curl, CURLOPT_POSTFIELDS, xml);
-		curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
-		curl_easy_setopt(curl, CURLOPT_WRITEDATA, &data);
+		w_curl_easy_setopt(curl, CURLOPT_URL, url);
+		w_curl_easy_setopt(curl, CURLOPT_POSTFIELDS, xml);
+		w_curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
+		w_curl_easy_setopt(curl, CURLOPT_WRITEDATA, &data);
 		long http_code = 0;
 		res = curl_easy_perform(curl);
 		int resp = -1;
@@ -120,6 +130,7 @@ int post(char*  url, char* xml, char** response){
 		return resp;
 	}
 
+error:
 	free(data.data);
 	return  -1;
 }
