@@ -111,12 +111,16 @@
 #include "t_funcs.h"
 #include "t_reply.h"
 #include "t_cancel.h"
+#include "t_stats.h"
 
 
 static struct timer_table *timertable=0;
 static unsigned int timer_sets = 0;
 static struct timer detached_timer; /* just to have a value to compare with*/
 
+/* currently processed branch */
+extern int _tm_branch_index;
+
 #define DETACHED_LIST (&detached_timer)
 
 #define is_in_timer_list2(_tl) ( (_tl)->timer_list &&  \
@@ -212,13 +216,13 @@ static void delete_cell( struct cell *p_cell, int unlock )
 		if (is_in_timer_list2(& p_cell->uac[i].local_cancel.retr_timer)) {
 			LM_ERR("transaction %p scheduled for deletion and"
 				" still on RETR/cancel (req %d), timeout %lld\n", p_cell, i,
-				p_cell->uac[i].request.retr_timer.time_out);
+				p_cell->uac[i].local_cancel.retr_timer.time_out);
 			abort();
 		}
 		if (is_in_timer_list2(& p_cell->uac[i].local_cancel.fr_timer)) {
 			LM_ERR("transaction %p scheduled for deletion and"
 				" still on FR/cancel (req %d), timeout %lld\n", p_cell, i,
-				p_cell->uac[i].request.fr_timer.time_out);
+				p_cell->uac[i].local_cancel.fr_timer.time_out);
 			abort();
 		}
 	}
@@ -240,16 +244,14 @@ static void delete_cell( struct cell *p_cell, int unlock )
 	}
 }
 
-
+/* used to inject a negative reply in behalf of a certian T branch; as it is
+ * a non-200OK reply, it cannot lead to a cancelling scenario */
 static void fake_reply(struct cell *t, int branch, int code )
 {
-	branch_bm_t cancel_bitmap;
-	short do_cancel_branch;
+	branch_bm_t cancel_bitmap = 0;
 	enum rps reply_status;
 
-	do_cancel_branch = is_invite(t) && should_cancel_branch(t, branch);
-
-	cancel_bitmap=do_cancel_branch ? 1<<branch : 0;
+	_tm_branch_index = branch;
 	if ( is_local(t) ) {
 		reply_status=local_reply( t, FAKED_REPLY, branch,
 					  code, &cancel_bitmap );
@@ -260,6 +262,12 @@ static void fake_reply(struct cell *t, int branch, int code )
 		reply_status=relay_reply( t, FAKED_REPLY, branch, code,
 			&cancel_bitmap );
 	}
+	_tm_branch_index = 0;
+	/* again, a final negative reply on a branch will never lead to a
+	 * situation to cancel other existing branches, so the
+	 * cancel_bitmap should be empty here (we use it as a dummy holder), so
+	 * no need trigger an UACs cancelling
+	 */
 }
 
 
@@ -287,19 +295,38 @@ inline static void retransmission_handler( struct timer_link *retr_tl )
 			LM_DBG("retransmission_handler : request resending"
 				" (t=%p, %.9s ... )\n", r_buf->my_T, r_buf->buffer.s);
 			set_t(r_buf->my_T);
-			SEND_BUFFER( r_buf );
-			/*if (SEND_BUFFER( r_buf )==-1) {
+			if (SEND_BUFFER( r_buf )==0) {
+				if ( has_tran_tmcbs( r_buf->my_T, TMCB_MSG_SENT_OUT) ) {
+					set_extra_tmcb_params( &r_buf->buffer, &r_buf->dst);
+					run_trans_callbacks( TMCB_MSG_SENT_OUT, r_buf->my_T,
+						r_buf->my_T->uas.request, 0, 0);
+				}
+			/*} else {
 				reset_timer( &r_buf->fr_timer );
 				fake_reply(r_buf->my_T, r_buf->branch, 503 );
 				return;
-			}*/
+			} */
+			}
+
 			set_t(T_UNDEFINED);
+			switch(r_buf->retr_list) {
+			case RT_T1_TO_1:
+				if_update_stat(tm_enable_stats, tm_retran_req_T11, 1); break;
+			case RT_T1_TO_2:
+				if_update_stat(tm_enable_stats, tm_retran_req_T12, 1); break;
+			case RT_T1_TO_3:
+				if_update_stat(tm_enable_stats, tm_retran_req_T13, 1); break;
+			case RT_T2:
+				if_update_stat(tm_enable_stats, tm_retran_req_T2, 1);break;
+			default:;
+			}
 	} else {
 			LM_DBG("retransmission_handler : reply resending "
 				"(t=%p, %.9s ... )\n", r_buf->my_T, r_buf->buffer.s);
 			set_t(r_buf->my_T);
 			t_retransmit_reply(r_buf->my_T);
 			set_t(T_UNDEFINED);
+			if_update_stat(tm_enable_stats, tm_retran_rpl_T2, 1);
 	}
 
 	id = r_buf->retr_list;
@@ -323,6 +350,7 @@ inline static void final_response_handler( struct timer_link *fr_tl )
 	context_p old_ctx;
 	struct retr_buf* r_buf;
 	struct cell *t;
+	branch_bm_t cancel_bitmap;
 
 	if (fr_tl==0){
 		/* or BUG?, ignoring it for now */
@@ -372,19 +400,30 @@ inline static void final_response_handler( struct timer_link *fr_tl )
 		my_ctx = context_alloc(CONTEXT_GLOBAL);
 		if (my_ctx==NULL) {
 			LM_ERR("failed to alloc new ctx in pkg\n");
+			return; /* otherwise we'll get a crash next instruction */
 		}
 	}
 	memset( my_ctx, 0, context_size(CONTEXT_GLOBAL) );
-	current_processing_ctx = my_ctx;
+	set_global_context(my_ctx);
 	/* set the T context too */
 	set_t( t );
 
 	/* out-of-lock do the cancel I/O */
 	if (is_invite(t) && should_cancel_branch(t, r_buf->branch) ) {
-		set_cancel_extra_hdrs( CANCEL_REASON_SIP_480, sizeof(CANCEL_REASON_SIP_480)-1);
-		cancel_branch(t, r_buf->branch );
+		cancel_bitmap =  1 << r_buf->branch ;
+		set_cancel_extra_hdrs( CANCEL_REASON_SIP_480,
+			sizeof(CANCEL_REASON_SIP_480)-1);
+		cancel_uacs(t, cancel_bitmap );
 		set_cancel_extra_hdrs( NULL, 0);
 	}
+
+	switch(r_buf->retr_list) {
+	case FR_TIMER_LIST:
+		if_update_stat(tm_enable_stats, tm_timeout_fr, 1); break;
+	case FR_INV_TIMER_LIST:
+		if_update_stat(tm_enable_stats, tm_timeout_fr_inv, 1); break;
+	default:;
+	}
 	/* lock reply processing to determine how to proceed reliably */
 	LOCK_REPLIES( t );
 	LM_DBG("Cancel sent out, sending 408 (%p)\n", t);
@@ -396,7 +435,7 @@ inline static void final_response_handler( struct timer_link *fr_tl )
 	else
 		context_destroy(CONTEXT_GLOBAL, my_ctx);
 	/* switch back to the old context */
-	current_processing_ctx = old_ctx;
+	set_global_context(old_ctx);
 	/* reset the T context */
 	init_t();
 
@@ -582,8 +621,9 @@ struct timer_table *tm_init_timers( unsigned int sets )
 			init_timer_list( set, i );
 
 		/* exclusion timer */
-		if ((timertable[set].ex_lock = lock_init_rw()) == NULL) {
-			LM_CRIT("failed to init timer RW lock\n");
+		if ((timertable[set].ex_lock = lock_alloc()) == NULL ||
+		lock_init(timertable[set].ex_lock) == NULL) {
+			LM_CRIT("failed to init timer lock\n");
 			goto error0;
 		}
 
@@ -613,7 +653,7 @@ void free_timer_table(void)
 		for ( i=0 ; i<timer_sets*NR_OF_TIMER_LISTS ; i++ )
 			release_timerlist_lock( &timertable->timers[i] );
 		for ( i=0 ; i<timer_sets ; i++ )
-			lock_destroy_rw( timertable[i].ex_lock );
+			lock_destroy( timertable[i].ex_lock );
 		shm_free(timertable);
 	}
 }
@@ -930,12 +970,18 @@ end:
 
 
 /* similar to set_timer, except it allows only one-time
-   timer setting and all later attempts are ignored */
-void set_1timer( struct timer_link *new_tl, enum lists list_id,
+ * timer setting and all later attempts are ignored
+ * Returned values:
+ *    0 if the linker was actually added to the list
+ *   -1 if the linker was not added as it was already found in the list;
+ *      do not consider this an error, but a NOP.
+ */
+int set_1timer( struct timer_link *new_tl, enum lists list_id,
 												utime_t* ext_timeout )
 {
 	utime_t timeout;
 	struct timer* list;
+	int ret = -1;
 
 
 	if (list_id>=NR_OF_TIMER_LISTS) {
@@ -943,7 +989,7 @@ void set_1timer( struct timer_link *new_tl, enum lists list_id,
 #ifdef EXTRA_DEBUG
 		abort();
 #endif
-		return;
+		return ret;
 	}
 
 	if (!ext_timeout) {
@@ -958,8 +1004,11 @@ void set_1timer( struct timer_link *new_tl, enum lists list_id,
 	if (!new_tl->time_out) {
 		insert_timer_unsafe( list, new_tl, timeout +
 			((timer_id2type[list_id]==UTIME_TYPE)?get_uticks():get_ticks()));
+		ret = 0;
 	}
 	unlock(list->mutex);
+
+	return ret;
 }
 
 
@@ -1049,9 +1098,12 @@ static void unlink_timers( struct cell *t )
 void timer_routine(unsigned int ticks , void *set)
 {
 	struct timer_link *tl, *tmp_tl;
+	struct timespec begin;
 	int                id;
 
-	lock_start_write( timertable[(long)set].ex_lock );
+	clock_gettime(CLOCK_REALTIME, &begin);
+
+	lock_get( timertable[(long)set].ex_lock );
 
 	for( id=0 ; id<RT_T1_TO_1 ; id++ )
 	{
@@ -1073,7 +1125,11 @@ void timer_routine(unsigned int ticks , void *set)
 				break;
 		}
 	}
-	lock_stop_write( timertable[(long)set].ex_lock );
+	lock_release( timertable[(long)set].ex_lock );
+
+	clock_check_diff((double)TM_TIMER_ITV_S*1e9 * TM_TIMER_LOAD_WARN,
+	    "now at %d%%+ capacity, inuse_transactions: %lu", (int)(TM_TIMER_LOAD_WARN*100),
+	    (unsigned long)get_stat_val(tm_trans_inuse));
 }
 
 
@@ -1081,9 +1137,12 @@ void timer_routine(unsigned int ticks , void *set)
 void utimer_routine(utime_t uticks , void *set)
 {
 	struct timer_link *tl, *tmp_tl;
+	struct timespec begin;
 	int                id;
 
-	lock_start_write( timertable[(long)set].ex_lock );
+	clock_gettime(CLOCK_REALTIME, &begin);
+
+	lock_get( timertable[(long)set].ex_lock );
 
 	for( id=RT_T1_TO_1 ; id<NR_OF_TIMER_LISTS ; id++ )
 	{
@@ -1101,6 +1160,10 @@ void utimer_routine(utime_t uticks , void *set)
 				break;
 		}
 	}
-	lock_stop_write( timertable[(long)set].ex_lock );
+	lock_release( timertable[(long)set].ex_lock );
+
+	clock_check_diff((double)TM_UTIMER_ITV_US*1000 * TM_TIMER_LOAD_WARN,
+	    "now at %d%%+ capacity, inuse_transactions: %lu", (int)(TM_TIMER_LOAD_WARN*100),
+	    (unsigned long)get_stat_val(tm_trans_inuse));
 }
 
