@@ -234,7 +234,7 @@ static inline void set_sock_reuseport(int s)
 /*! \brief Set all socket/fd options:  disable nagle, tos lowdelay,
  * non-blocking
  * \return -1 on error */
-int tcp_init_sock_opt(int s, const struct tcp_conn_profile *prof, enum si_flags socketflags)
+int tcp_init_sock_opt(int s, const struct tcp_conn_profile *prof, enum si_flags socketflags, int sock_tos)
 {
 	int flags;
 	int optval;
@@ -247,10 +247,12 @@ int tcp_init_sock_opt(int s, const struct tcp_conn_profile *prof, enum si_flags
 	}
 #endif
 	/* tos*/
-	optval = tos;
-	if (setsockopt(s, IPPROTO_IP, IP_TOS, (void*)&optval,sizeof(optval)) ==-1){
-		LM_WARN("setsockopt tos: %s\n",	strerror(errno));
-		/* continue since this is not critical */
+	optval = (sock_tos > 0) ? sock_tos : tos;
+	if (optval > 0) {
+		if (setsockopt(s, IPPROTO_IP, IP_TOS, (void*)&optval,sizeof(optval)) ==-1){
+			LM_WARN("setsockopt tos: %s\n",	strerror(errno));
+			/* continue since this is not critical */
+		}
 	}
 
 	if (probe_max_sock_buff(s,1,MAX_SEND_BUFFER_SIZE,BUFFER_INCREMENT)) {
@@ -323,6 +325,17 @@ static int send2worker(struct tcp_connection* tcpconn,int rw)
 
 /* initializes an already defined TCP listener */
 int tcp_init_listener(struct socket_info *si)
+{
+	union sockaddr_union* addr = &si->su;
+	if (init_su(addr, &si->address, si->port_no)<0){
+		LM_ERR("could no init sockaddr_union\n");
+		return -1;
+	}
+	return 0;
+}
+
+/* binding an defined TCP listener */
+int tcp_bind_listener(struct socket_info *si)
 {
 	union sockaddr_union* addr;
 	int optval;
@@ -342,10 +355,6 @@ int tcp_init_listener(struct socket_info *si)
 #endif
 
 	addr = &si->su;
-	if (init_su(addr, &si->address, si->port_no)<0){
-		LM_ERR("could no init sockaddr_union\n");
-		goto error;
-	}
 	si->socket = socket(AF2PF(addr->s.sa_family), SOCK_STREAM, 0);
 	if (si->socket==-1){
 		LM_ERR("socket failed with [%s]\n", strerror(errno));
@@ -375,11 +384,13 @@ int tcp_init_listener(struct socket_info *si)
 	}
 #endif
 	/* tos */
-	optval = tos;
-	if (setsockopt(si->socket, IPPROTO_IP, IP_TOS, (void*)&optval,
-	sizeof(optval)) ==-1){
-		LM_WARN("setsockopt tos: %s\n", strerror(errno));
-		/* continue since this is not critical */
+	optval = (si->tos > 0) ? si->tos : tos;
+	if (optval > 0) {
+		if (setsockopt(si->socket, IPPROTO_IP, IP_TOS, (void*)&optval,
+		sizeof(optval)) ==-1){
+			LM_WARN("setsockopt tos: %s\n", strerror(errno));
+			/* continue since this is not critical */
+		}
 	}
 
 	if (probe_max_sock_buff(si->socket,1,MAX_SEND_BUFFER_SIZE,
@@ -1064,6 +1075,8 @@ static inline void tcpconn_destroy(struct tcp_connection* tcpconn)
 		/* force timeout */
 		tcpconn->lifetime=0;
 		tcpconn->state=S_CONN_BAD;
+		sh_log(tcpconn->hist, TCP_DEL_DELAY, "tcpconn_destroy delayed, (%d)",
+			tcpconn->refcnt);
 		LM_DBG("delaying (%p, flags %04x) ref = %d ...\n",
 				tcpconn, tcpconn->flags, tcpconn->refcnt);
 
@@ -1116,7 +1129,7 @@ static inline int handle_new_connect(const struct socket_info* si)
 	}
 
 	tcp_con_get_profile(&su, &si->su, si->proto, &prof);
-	if (tcp_init_sock_opt(new_sock, &prof, si->flags)<0){
+	if (tcp_init_sock_opt(new_sock, &prof, si->flags, si->tos)<0){
 		LM_ERR("tcp_init_sock_opt failed\n");
 		close(new_sock);
 		return 1; /* success, because the accept was successful */
@@ -1373,6 +1386,9 @@ inline static int handle_tcp_worker(struct tcp_worker* tcp_c, int fd_i)
 			tcpconn->flags&=~F_CONN_REMOVED_WRITE;
 			break;
 		case CONN_ERROR_TCPW:
+			LM_ERR("TCP_DBG - main: received conn %p / %u as faulty "
+				"(state %d, rfcnt=%d)\n", tcpconn, tcpconn->id,
+				tcpconn->state, tcpconn->refcnt);
 		case CONN_DESTROY:
 		case CONN_EOF:
 			/* WARNING: this will auto-dec. refcnt! */
@@ -1526,8 +1542,11 @@ inline static int handle_worker(struct process_table* p, int fd_i)
 			tcpconn->flags&=~F_CONN_REMOVED_WRITE;
 			break;
 		case ASYNC_WRITE_GENW:
+			sh_log(tcpconn->hist,TCP_UNREF,"ASYNC_WRITE_GENW, (%d)",
+				tcpconn->refcnt);
 			if (tcpconn->state==S_CONN_BAD){
 				tcpconn->lifetime=0;
+				tcpconn_put(tcpconn);
 				break;
 			}
 			tcpconn_put(tcpconn);
@@ -1687,6 +1706,12 @@ static void tcp_main_server(void)
 		if ( is_tcp_based_proto(n) )
 			for( sif=protos[n].listeners ; sif ; sif=sif->next ) {
 				struct socket_info* si = &sif->socket_info;
+				if (protos[n].tran.bind_listener && protos[n].tran.bind_listener(si)<0) {
+					LM_ERR("failed to bind listener [%.*s], proto %s\n",
+							si->name.len, si->name.s,
+							protos[n].name );
+					goto error;
+				}
 				if ( (si->socket!=-1) &&
 				reactor_add_reader( si->socket, F_TCP_LISTENER,
 				RCT_PRIO_NET, si)<0 ) {
@@ -2278,6 +2303,7 @@ mi_response_t *mi_tcp_list_conns(const mi_params_t *params,
 					/* add one node for each conn */
 					add_mi_number( conn_item, MI_SSTR("Alias port"),
 						conn->con_aliases[j].port );
+
 			}
 		}
 
