@@ -203,6 +203,7 @@ static int proto_tcp_init(struct proto_info *pi)
 	pi->default_port		= tcp_port;
 
 	pi->tran.init_listener	= proto_tcp_init_listener;
+	pi->tran.bind_listener	= tcp_bind_listener;
 	pi->tran.send			= proto_tcp_send;
 	pi->tran.dst_attr		= tcp_conn_fcntl;
 
@@ -449,7 +450,7 @@ static int proto_tcp_send(const struct socket_info* send_sock,
 				 * flow now (the actual write will be done when
 				 * connect will be completed */
 				LM_DBG("Successfully started async connection \n");
-				sh_log(c->hist, TCP_SEND2MAIN, "send 1, (%d)", c->refcnt);
+				sh_log(c->hist, TCP_RELEASED, "send 1, (%d)", c->refcnt);
 				tcp_conn_release(c, 0);
 				return len;
 			}
@@ -525,7 +526,7 @@ static int proto_tcp_send(const struct socket_info* send_sock,
 				LM_ERR("Failed to add another write chunk to %p\n",c);
 				/* we failed due to internal errors - put the
 				 * connection back */
-				sh_log(c->hist, TCP_SEND2MAIN, "send 2, (%d)", c->refcnt);
+				sh_log(c->hist, TCP_RELEASED, "send 2, (%d)", c->refcnt);
 				tcp_conn_release(c, 0);
 				return -1;
 			}
@@ -536,12 +537,14 @@ static int proto_tcp_send(const struct socket_info* send_sock,
 			send_sock->last_real_ports->remote = c->rcv.src_port;
 
 			/* we successfully added our write chunk - success */
-			sh_log(c->hist, TCP_SEND2MAIN, "send 3, (%d)", c->refcnt);
+			sh_log(c->hist, TCP_RELEASED, "send 3, (%d)", c->refcnt);
 			tcp_conn_release(c, 0);
 			return len;
 		} else {
-			/* return error, nothing to do about it */
-			sh_log(c->hist, TCP_SEND2MAIN, "send 4, (%d)", c->refcnt);
+			/* the FD transfer failed (we have an established conn, 
+			 * but returned fd is -1) -> leave the conn alone, return error
+			 * for the write op, nothing to do about it */
+			sh_log(c->hist, TCP_RELEASED, "send 4, (%d)", c->refcnt);
 			tcp_conn_release(c, 0);
 			return -1;
 		}
@@ -564,12 +567,12 @@ send_it:
 	LM_DBG("after write: c= %p n/len=%d/%d fd=%d\n",c, n, len, fd);
 	/* LM_DBG("buf=\n%.*s\n", (int)len, buf); */
 	if (n<0){
-		LM_ERR("failed to send\n");
+		LM_ERR("failed to send on conn %p / %u\n", c, c->id);
 		c->state=S_CONN_BAD;
 		if (c->proc_id != process_no)
 			close(fd);
 
-		sh_log(c->hist, TCP_SEND2MAIN, "send 5, (%d)", c->refcnt);
+		sh_log(c->hist, TCP_RELEASED, "send 5, (%d)", c->refcnt);
 		tcp_conn_release(c, 0);
 		return -1;
 	}
@@ -584,7 +587,8 @@ send_it:
 	send_sock->last_real_ports->local = c->rcv.dst_port;
 	send_sock->last_real_ports->remote = c->rcv.src_port;
 
-	sh_log(c->hist, TCP_SEND2MAIN, "send 6, (%d, async: %d)", c->refcnt, n < len);
+	if (n==len)
+		sh_log(c->hist, TCP_RELEASED, "send 6, (%d, async: %d)", c->refcnt, n < len);
 	tcp_conn_release(c, (n<len)?1:0/*pending data in async mode?*/ );
 	return n;
 }
