@@ -52,6 +52,7 @@
 #include "../core_stats.h"
 #include "../errinfo.h"
 #include "../dset.h"
+#include "../sdp_ops.h"
 #include "parse_hname2.h"
 #include "parse_uri.h"
 #include "parse_content.h"
@@ -598,11 +599,10 @@ int parse_headers_aux(struct sip_msg* msg, hdr_flags_t flags, int next, int sip_
 							msg->via2=msg->via1->next;
 							msg->parsed_flag|=HDR_VIA2_F;
 						}
-					}else if (msg->via2==0){
-						msg->h_via2=hf;
+					} else
+					if (msg->via2==0) {
 						msg->via2=hf->parsed;
 						msg->parsed_flag|=HDR_VIA2_F;
-						LM_DBG("parse_headers: this is the second via\n");
 					}
 				}
 				break;
@@ -982,6 +982,8 @@ void free_sip_msg(struct sip_msg* msg)
 		free_lump_list(msg->add_rm);
 	if (msg->body_lumps)
 		free_lump_list(msg->body_lumps);
+	if (msg->sdp_ops )
+		free_sdp_ops(msg->sdp_ops);
 	if (msg->reply_lump)
 		free_reply_lump(msg->reply_lump);
 	if (msg->body )
